/* nickserv.h - Nick/authentiction service
 * Copyright 2000-2001 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: nickserv.h,v 1.24.2.8 2001/08/14 01:19:27 entrope Exp $
 */

#ifndef _nickserv_h
#define _nickserv_h

#include "common.h"

#define NICKSERV_HANDLE_LEN NICKLEN

struct userNode;
struct string_list;

/* HI_FLAG_* go into handle_info.flags */
#define HI_FLAG_OPER_SUSPENDED 0x00000001
#define HI_FLAG_USE_PRIVMSG    0x00000002
#define HI_FLAG_HELPER         0x00000004
#define HI_FLAG_HELPING        0x00000008
#define HI_FLAG_SUSPENDED      0x00000010
#define HI_FLAG_MIRC_COLOR     0x00000020
#define HI_FLAG_FROZEN         0x00000040
#define HI_FLAG_NODELETE       0x00000080
/* Flag characters for the above.  First char is LSB, etc. */
#define HANDLE_FLAGS "Sphgscfn"

/* HI_STYLE_* go into handle_info.userlist_style */
#define HI_STYLE_DEF	       'd'
#define HI_STYLE_ZOOT	       'Z'

#define HI_DEFAULT_FLAGS       (HI_FLAG_MIRC_COLOR)
#define HI_DEFAULT_STYLE       HI_STYLE_DEF

#define HANDLE_FLAGGED(hi, tok) ((hi)->flags & HI_FLAG_##tok)
#define HANDLE_SET_FLAG(hi, tok) ((hi)->flags |= HI_FLAG_##tok)
#define HANDLE_TOGGLE_FLAG(hi, tok) ((hi)->flags ^= HI_FLAG_##tok)
#define HANDLE_CLEAR_FLAG(hi, tok) ((hi)->flags &= ~HI_FLAG_##tok)

#define IsHelper(user) (user->handle_info && HANDLE_FLAGGED(user->handle_info, HELPER))
#define IsHelping(user) (user->handle_info && HANDLE_FLAGGED(user->handle_info, HELPING))
#define IsPrivileged(user) (IsHelping(user) || IsOper(user))

struct handle_info {
    struct nick_info *nicks; /* linked list of owned nicks */
    struct string_list *masks;
    struct userNode *users;
    struct chanList *channels;
    unsigned long flags;
    unsigned int opserv_level;
    unsigned int screen_width;
    time_t registered, lastseen;
    char handle[NICKSERV_HANDLE_LEN+1];
    char passwd[MD5_CRYPT_LENGTH+1];
    char info[MAXLEN+1];
    char userlist_style;
};

struct nick_info {
    struct handle_info *owner;
    struct nick_info *next; /* next nick owned by same handle */
    char nick[NICKLEN+1];
};

void init_nickserv(const char *nick);
struct handle_info *get_handle_info(const char *handle);
struct handle_info *smart_get_handle_info(struct userNode *service, struct userNode *user, const unsigned char *name);
struct nick_info *get_nick_info(const char *nick);

/* auth_funcs are called when a user gets a new handle_info.  They are
 * called after it has been determined that the user may authenticate
 * under the new handle, but *before* their handle_info is changed --
 * so user->handle_info still points to their previous handle (if they
 * were previously authenticated). */
typedef void (*auth_func_t)(struct userNode *user, struct handle_info *new_handle);
void reg_auth_func(auth_func_t func);

/* unreg_funcs are called right before a handle is unregistered.
 * `user' is the person who caused the handle to be unregistered (either a
 * client authed to the handle, or an oper). */
typedef void (*unreg_func_t)(struct userNode *user, struct handle_info *handle);
void reg_unreg_func(unreg_func_t func);

#endif
